@@ -24,6 +24,7 @@
 
 #include <boost/filesystem/config.hpp>
 #include <boost/filesystem/path.hpp>
+#include <boost/filesystem/operations.hpp>  // for filesystem_error
 #include <boost/scoped_array.hpp>
 #include <boost/system/error_code.hpp>
 #include <boost/assert.hpp>
@@ -37,7 +38,7 @@
 # include <windows.h>
 #elif defined(macintosh) || defined(__APPLE__) || defined(__APPLE_CC__) \
  || defined(__CYGWIN__) \
- || defined(__FreeBSD__) || defined(__OPEN_BSD__) || defined(__HAIKU__)
+ || defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__HAIKU__)
 # include <boost/filesystem/detail/utf8_codecvt_facet.hpp>
 #endif
 
@@ -73,12 +74,10 @@ namespace
 
 # ifdef BOOST_WINDOWS_API
 
-  const wchar_t separator = L'/';
   const wchar_t* const separators = L"/\\";
   const wchar_t* separator_string = L"/";
   const wchar_t* preferred_separator_string = L"\\";
   const wchar_t colon = L':';
-  const wchar_t dot = L'.';
   const wchar_t questionmark = L'?';
 
   inline bool is_letter(wchar_t c)
@@ -88,23 +87,12 @@ namespace
 
 # else
 
-  const char separator = '/';
   const char* const separators = "/";
   const char* separator_string = "/";
   const char* preferred_separator_string = "/";
-  const char dot = '.';
 
 # endif
 
-  inline bool is_separator(fs::path::value_type c)
-  {
-    return c == separator
-#     ifdef BOOST_WINDOWS_API
-      || c == path::preferred_separator
-#     endif
-      ;
-  }
-
   bool is_root_separator(const string_type& str, size_type pos);
     // pos is position of the separator
 
@@ -145,13 +133,13 @@ namespace filesystem
     if (this == &p)  // self-append
     {
       path rhs(p);
-      if (!is_separator(rhs.m_pathname[0]))
+      if (!detail::is_directory_separator(rhs.m_pathname[0]))
         m_append_separator_if_needed();
       m_pathname += rhs.m_pathname;
     }
     else
     {
-      if (!is_separator(*p.m_pathname.begin()))
+      if (!detail::is_directory_separator(*p.m_pathname.begin()))
         m_append_separator_if_needed();
       m_pathname += p.m_pathname;
     }
@@ -166,13 +154,13 @@ namespace filesystem
       && ptr < m_pathname.data() + m_pathname.size())  // overlapping source
     {
       path rhs(ptr);
-      if (!is_separator(rhs.m_pathname[0]))
+      if (!detail::is_directory_separator(rhs.m_pathname[0]))
         m_append_separator_if_needed();
       m_pathname += rhs.m_pathname;
     }
     else
     {
-      if (!is_separator(*ptr))
+      if (!detail::is_directory_separator(*ptr))
         m_append_separator_if_needed();
       m_pathname += ptr;
     }
@@ -217,7 +205,7 @@ namespace filesystem
 #     ifdef BOOST_WINDOWS_API
       *(m_pathname.end()-1) != colon && 
 #     endif
-      !is_separator(*(m_pathname.end()-1)))
+      !detail::is_directory_separator(*(m_pathname.end()-1)))
     {
       string_type::size_type tmp(m_pathname.size());
       m_pathname += preferred_separator;
@@ -257,7 +245,8 @@ namespace filesystem
 
   path&  path::remove_trailing_separator()
   {
-    if (!m_pathname.empty() && is_separator(m_pathname[m_pathname.size() - 1]))
+    if (!m_pathname.empty()
+      && detail::is_directory_separator(m_pathname[m_pathname.size() - 1]))
       m_pathname.erase(m_pathname.size() - 1);
     return *this;
   }
@@ -294,8 +283,8 @@ namespace filesystem
     return (itr.m_pos != m_pathname.size()
       && (
           (itr.m_element.m_pathname.size() > 1
-            && is_separator(itr.m_element.m_pathname[0])
-            && is_separator(itr.m_element.m_pathname[1])
+            && detail::is_directory_separator(itr.m_element.m_pathname[0])
+            && detail::is_directory_separator(itr.m_element.m_pathname[1])
    )
 #       ifdef BOOST_WINDOWS_API
         || itr.m_element.m_pathname[itr.m_element.m_pathname.size()-1] == colon
@@ -319,7 +308,7 @@ namespace filesystem
     iterator itr(begin());
 
     for (; itr.m_pos != m_pathname.size()
-      && (is_separator(itr.m_element.m_pathname[0])
+      && (detail::is_directory_separator(itr.m_element.m_pathname[0])
 #     ifdef BOOST_WINDOWS_API
       || itr.m_element.m_pathname[itr.m_element.m_pathname.size()-1] == colon
 #     endif
@@ -333,14 +322,14 @@ namespace filesystem
     size_type end_pos(filename_pos(m_pathname, m_pathname.size()));
 
     bool filename_was_separator(m_pathname.size()
-      && is_separator(m_pathname[end_pos]));
+      && detail::is_directory_separator(m_pathname[end_pos]));
 
     // skip separators unless root directory
     size_type root_dir_pos(root_directory_start(m_pathname, end_pos));
     for (; 
       end_pos > 0
       && (end_pos-1) != root_dir_pos
-      && is_separator(m_pathname[end_pos-1])
+      && detail::is_directory_separator(m_pathname[end_pos-1])
       ;
       --end_pos) {}
 
@@ -362,7 +351,7 @@ namespace filesystem
     size_type pos(filename_pos(m_pathname, m_pathname.size()));
     return (m_pathname.size()
               && pos
-              && is_separator(m_pathname[pos])
+              && detail::is_directory_separator(m_pathname[pos])
               && !is_root_separator(m_pathname, pos))
       ? detail::dot_path()
       : path(m_pathname.c_str() + pos);
@@ -388,11 +377,47 @@ namespace filesystem
       : path(name.m_pathname.c_str() + pos);
   }
 
-  // m_normalize  ----------------------------------------------------------------------//
+  //  lexical operations  --------------------------------------------------------------//
 
-  path& path::m_normalize()
+  namespace detail
   {
-    if (m_pathname.empty()) return *this;
+    // C++14 provide a mismatch algorithm with four iterator arguments(), but earlier
+    // standard libraries didn't, so provide this needed functionality.
+    inline
+    std::pair<path::iterator, path::iterator> mismatch(path::iterator it1,
+      path::iterator it1end, path::iterator it2, path::iterator it2end)
+    {
+      for (; it1 != it1end && it2 != it2end && *it1 == *it2;)
+      {
+        ++it1;
+        ++it2;
+      }
+      return std::make_pair(it1, it2);
+    }
+  }
+
+  path path::lexically_relative(const path& base) const
+  {
+    std::pair<path::iterator, path::iterator> mm
+      = detail::mismatch(begin(), end(), base.begin(), base.end());
+    if (mm.first == begin() && mm.second == base.begin())
+      return path();
+    if (mm.first == end() && mm.second == base.end())
+      return detail::dot_path();
+    path tmp;
+    for (; mm.second != base.end(); ++mm.second)
+      tmp /= detail::dot_dot_path();
+    for (; mm.first != end(); ++mm.first)
+      tmp /= *mm.first;
+    return tmp;
+  }
+
+  //  normal  --------------------------------------------------------------------------//
+
+  path path::lexically_normal() const
+  {
+    if (m_pathname.empty())
+      return *this;
       
     path temp;
     iterator start(begin());
@@ -428,21 +453,26 @@ namespace filesystem
           )
         {
           temp.remove_filename();
-          // if not root directory, must also remove "/" if any
-          if (temp.m_pathname.size() > 0
-            && temp.m_pathname[temp.m_pathname.size()-1]
-              == separator)
-          {
-            string_type::size_type rds(
-              root_directory_start(temp.m_pathname, temp.m_pathname.size()));
-            if (rds == string_type::npos
-              || rds != temp.m_pathname.size()-1) 
-              { temp.m_pathname.erase(temp.m_pathname.size()-1); }
-          }
+          //// if not root directory, must also remove "/" if any
+          //if (temp.native().size() > 0
+          //  && temp.native()[temp.native().size()-1]
+          //    == separator)
+          //{
+          //  string_type::size_type rds(
+          //    root_directory_start(temp.native(), temp.native().size()));
+          //  if (rds == string_type::npos
+          //    || rds != temp.native().size()-1) 
+          //  {
+          //    temp.m_pathname.erase(temp.native().size()-1);
+          //  }
+          //}
 
           iterator next(itr);
           if (temp.empty() && ++next != stop
-            && next == last && *last == detail::dot_path()) temp /= detail::dot_path();
+            && next == last && *last == detail::dot_path())
+          {
+            temp /= detail::dot_path();
+          }
           continue;
         }
       }
@@ -450,9 +480,9 @@ namespace filesystem
       temp /= *itr;
     };
 
-    if (temp.empty()) temp /= detail::dot_path();
-    m_pathname = temp.m_pathname;
-    return *this;
+    if (temp.empty())
+      temp /= detail::dot_path();
+    return temp;
   }
 
 }  // namespace filesystem
@@ -472,11 +502,11 @@ namespace
   bool is_root_separator(const string_type & str, size_type pos)
     // pos is position of the separator
   {
-    BOOST_ASSERT_MSG(!str.empty() && is_separator(str[pos]),
+    BOOST_ASSERT_MSG(!str.empty() && fs::detail::is_directory_separator(str[pos]),
       "precondition violation");
 
     // subsequent logic expects pos to be for leftmost slash of a set
-    while (pos > 0 && is_separator(str[pos-1]))
+    while (pos > 0 && fs::detail::is_directory_separator(str[pos-1]))
       --pos;
 
     //  "/" [...]
@@ -490,7 +520,8 @@ namespace
 # endif
 
     //  "//" name "/"
-    if (pos < 3 || !is_separator(str[0]) || !is_separator(str[1]))
+    if (pos < 3 || !fs::detail::is_directory_separator(str[0])
+      || !fs::detail::is_directory_separator(str[1]))
       return false;
 
     return str.find_first_of(separators, 2) == pos;
@@ -504,11 +535,11 @@ namespace
   {
     // case: "//"
     if (end_pos == 2 
-      && is_separator(str[0])
-      && is_separator(str[1])) return 0;
+      && fs::detail::is_directory_separator(str[0])
+      && fs::detail::is_directory_separator(str[1])) return 0;
 
     // case: ends in "/"
-    if (end_pos && is_separator(str[end_pos-1]))
+    if (end_pos && fs::detail::is_directory_separator(str[end_pos-1]))
       return end_pos-1;
     
     // set pos to start of last element
@@ -520,7 +551,7 @@ namespace
 #   endif
 
     return (pos == string_type::npos // path itself must be a filename (or empty)
-      || (pos == 1 && is_separator(str[0]))) // or net
+      || (pos == 1 && fs::detail::is_directory_separator(str[0]))) // or net
         ? 0 // so filename is entire string
         : pos + 1; // or starts after delimiter
   }
@@ -535,21 +566,21 @@ namespace
     // case "c:/"
     if (size > 2
       && path[1] == colon
-      && is_separator(path[2])) return 2;
+      && fs::detail::is_directory_separator(path[2])) return 2;
 #   endif
 
     // case "//"
     if (size == 2
-      && is_separator(path[0])
-      && is_separator(path[1])) return string_type::npos;
+      && fs::detail::is_directory_separator(path[0])
+      && fs::detail::is_directory_separator(path[1])) return string_type::npos;
 
 #   ifdef BOOST_WINDOWS_API
     // case "\\?\"
     if (size > 4
-      && is_separator(path[0])
-      && is_separator(path[1])
+      && fs::detail::is_directory_separator(path[0])
+      && fs::detail::is_directory_separator(path[1])
       && path[2] == questionmark
-      && is_separator(path[3]))
+      && fs::detail::is_directory_separator(path[3]))
     {
       string_type::size_type pos(path.find_first_of(separators, 4));
         return pos < size ? pos : string_type::npos;
@@ -558,16 +589,16 @@ namespace
 
     // case "//net {/}"
     if (size > 3
-      && is_separator(path[0])
-      && is_separator(path[1])
-      && !is_separator(path[2]))
+      && fs::detail::is_directory_separator(path[0])
+      && fs::detail::is_directory_separator(path[1])
+      && !fs::detail::is_directory_separator(path[2]))
     {
       string_type::size_type pos(path.find_first_of(separators, 2));
       return pos < size ? pos : string_type::npos;
     }
     
     // case "/"
-    if (size > 0 && is_separator(path[0])) return 0;
+    if (size > 0 && fs::detail::is_directory_separator(path[0])) return 0;
 
     return string_type::npos;
   }
@@ -591,22 +622,22 @@ namespace
     string_type::size_type cur(0);
     
     // deal with // [network]
-    if (size >= 2 && is_separator(src[0])
-      && is_separator(src[1])
+    if (size >= 2 && fs::detail::is_directory_separator(src[0])
+      && fs::detail::is_directory_separator(src[1])
       && (size == 2
-        || !is_separator(src[2])))
+        || !fs::detail::is_directory_separator(src[2])))
     { 
       cur += 2;
       element_size += 2;
     }
 
     // leading (not non-network) separator
-    else if (is_separator(src[0]))
+    else if (fs::detail::is_directory_separator(src[0]))
     {
       ++element_size;
       // bypass extra leading separators
       while (cur+1 < size
-        && is_separator(src[cur+1]))
+        && fs::detail::is_directory_separator(src[cur+1]))
       {
         ++cur;
         ++element_pos;
@@ -622,7 +653,7 @@ namespace
 #     ifdef BOOST_WINDOWS_API
       && src[cur] != colon
 #     endif
-      && !is_separator(src[cur]))
+      && !fs::detail::is_directory_separator(src[cur]))
     {
       ++cur;
       ++element_size;
@@ -731,12 +762,12 @@ namespace filesystem
 
     // both POSIX and Windows treat paths that begin with exactly two separators specially
     bool was_net(it.m_element.m_pathname.size() > 2
-      && is_separator(it.m_element.m_pathname[0])
-      && is_separator(it.m_element.m_pathname[1])
-      && !is_separator(it.m_element.m_pathname[2]));
+      && detail::is_directory_separator(it.m_element.m_pathname[0])
+      && detail::is_directory_separator(it.m_element.m_pathname[1])
+      && !detail::is_directory_separator(it.m_element.m_pathname[2]));
 
     // process separator (Windows drive spec is only case not a separator)
-    if (is_separator(it.m_path_ptr->m_pathname[it.m_pos]))
+    if (detail::is_directory_separator(it.m_path_ptr->m_pathname[it.m_pos]))
     {
       // detect root directory
       if (was_net
@@ -752,7 +783,7 @@ namespace filesystem
 
       // skip separators until it.m_pos points to the start of the next element
       while (it.m_pos != it.m_path_ptr->m_pathname.size()
-        && is_separator(it.m_path_ptr->m_pathname[it.m_pos]))
+        && detail::is_directory_separator(it.m_path_ptr->m_pathname[it.m_pos]))
         { ++it.m_pos; }
 
       // detect trailing separator, and treat it as ".", per POSIX spec
@@ -781,7 +812,7 @@ namespace filesystem
     // if at end and there was a trailing non-root '/', return "."
     if (it.m_pos == it.m_path_ptr->m_pathname.size()
       && it.m_path_ptr->m_pathname.size() > 1
-      && is_separator(it.m_path_ptr->m_pathname[it.m_pos-1])
+      && detail::is_directory_separator(it.m_path_ptr->m_pathname[it.m_pos-1])
       && !is_root_separator(it.m_path_ptr->m_pathname, it.m_pos-1) 
        )
     {
@@ -797,7 +828,7 @@ namespace filesystem
       ; 
       end_pos > 0
       && (end_pos-1) != root_dir_pos
-      && is_separator(it.m_path_ptr->m_pathname[end_pos-1])
+      && detail::is_directory_separator(it.m_path_ptr->m_pathname[end_pos-1])
       ;
       --end_pos) {}
 
@@ -833,7 +864,7 @@ namespace
   //  indicated the current code is roughly 9% slower than the previous code, and that
   //  seems a small price to pay for better code that is easier to use. 
 
-  inline std::locale default_locale()
+  std::locale default_locale()
   {
 # if defined(BOOST_WINDOWS_API)
     std::locale global_loc = std::locale();
@@ -869,7 +900,7 @@ namespace
 # endif
   }
 
-  inline std::locale& path_locale()
+  std::locale& path_locale()
   // std::locale("") construction, needed on non-Apple POSIX systems, can throw
   // (if environmental variables LC_MESSAGES or LANG are wrong, for example), so
   // path_locale() provides lazy initialization via a local static to ensure that any 
@@ -878,6 +909,10 @@ namespace
   // actually called, ensuring that an exception will only be thrown if std::locale("")
   // is really needed.
   {
+    // [locale] paragraph 6: Once a facet reference is obtained from a locale object by
+    // calling use_facet<>, that reference remains usable, and the results from member
+    // functions of it may be cached and re-used, as long as some locale object refers
+    // to that facet.
     static std::locale loc(default_locale());
 #ifdef BOOST_FILESYSTEM_DEBUG
     std::cout << "***** path_locale() called" << std::endl;
@@ -902,6 +937,7 @@ namespace filesystem
     std::cout << "***** path::codecvt() called" << std::endl;
 #endif
     BOOST_ASSERT_MSG(&path_locale(), "boost::filesystem::path locale initialization error");
+
     return std::use_facet<std::codecvt<wchar_t, char, std::mbstate_t> >(path_locale());
   }
 
